==> hermes-3.0.1.26.tar.gz: hermes-3.0.1.26/app/serializers/hermes/rex/tree_serializer.rb <==
# frozen_string_literal: true

module Hermes
  module Rex
    # Rex tree serializer
    class TreeSerializer < ActiveModel::Serializer
      def for_lessor?
        instance_options[:for_lessor] == true
      end

      attributes :from_element, :tree

      def class_target
        for_lessor? ? Hermes::Lessor : Hermes::Entity
      end

      def serializer_target
        for_lessor? ? Hermes::LessorSerializer : Hermes::EntitySerializer
      end

      def serialize(element, root = true)
        serializer_target.new(class_target.find_by(id: element[:_id]), root: root, scope: scope).to_h
      end

      def recursive_serialize(object)
        real_object = class_target.find_by(id: object[:_id])
        serialized_child = serializer_target.new(real_object, root: false, scope: scope).to_h

        target_depth = object[:query_depth] + 1
        if target_depth <= max_depth
          sub_children = children(target_depth).select do |potential_sub_child|
            potential_sub_child[:parent_id] == object[:_id]
          end
          serialized_child[:children] = sub_children.map do |sub_child|
            recursive_serialize(sub_child)
          end
        end
        serialized_child
      end

      def from_element
        serialize(object)
      end

      def max_depth
        @max_depth ||= object[:children].map { |element| element[:query_depth] }
                                        .uniq
                                        .max
      end

      def children(depth = -1)
        if depth == -1
          object[:children]
        elsif depth >= 0
          object[:children].select do |element|
            element[:query_depth] == depth
          end
        end
      end

      def tree
        # fonction qui itère sur chaque élément passé dans self.object, puis qui structure les éléments présents
        # dans chacun de leur champ d'agrégation children
        # retourne à la fin un mapping de l'arborescence structurée de l'arbre sous forme de liste
        #   le champ query_depth des children indique le niveau de profondeur de l'objet en rapport avec le document mongo
        #   on peut utiliser le champ parent_id et id des children pour gérer l'imbriquation des éléments
        # l'idée étanobjectt d'imbriquer de façon récursive ce qui est possible d'intégrer dans un champ children
        # exemple de retour :
        # [
        #   {id: n0, title: etp1, ..., children: [
        #     {id: n1, title: etp2, ..., children: [
        #       {id: n2, title: etp3, ..., children: [ ... ]}
        #     ]},
        #     {id: n3, title: etp4, ..., children: [
        #       {id: n4, title: etp5, ..., children: [ ... ]}
        #     ]}
        #   ]},
        #   {id: n5, title: etpobject6, ..., children: [
        #     {id: n6, title: etp7, ..., children: [
        #       {id: n7, title: etp8, ..., children: [ ... ]}
        #     ]},
        #     {id: n8, title: etp9, ..., children: [
        #       {id: n9, title: etp10, ..., children: [ ... ]}
        #     ]}
        #   ]},
        #   ...
        # ]
        children(0).map do |direct_child|
          recursive_serialize(direct_child)
        end
      end
    end
  end
end

==> hermes-5.6.0.18.tar.gz: hermes-5.6.0.18/app/serializers/hermes/rex/tree_serializer.rb <==
# frozen_string_literal: true

module Hermes
  module Rex
    # Rex tree serializer
    class TreeSerializer < ActiveModel::Serializer
      def for_lessor?
        @instance_options[:for_lessor] == true
      end

      attributes :from_element, :tree

      def class_target
        for_lessor? ? Hermes::Lessor : Hermes::Entity
      end

      def serializer_target
        for_lessor? ? Hermes::LessorSerializer : Hermes::EntitySerializer
      end

      def serialize(element, root = true)
        serializer_target.new(class_target.find_by(id: element[:_id]), root: root, scope: scope).to_h
      end

      def recursive_serialize(object)
        real_object = class_target.find_by(id: object[:_id])
        serialized_child = serializer_target.new(real_object, root: false, scope: scope).to_h

        target_depth = object[:query_depth] + 1
        if target_depth <= max_depth
          sub_children = children(target_depth).select do |potential_sub_child|
            potential_sub_child[:parent_id] == object[:_id]
          end
          serialized_child[:children] = sub_children.map do |sub_child|
            recursive_serialize(sub_child)
          end
        end
        serialized_child
      end

      def from_element
        serialize(object)
      end

      def max_depth
        @max_depth ||= object[:children].map { |element| element[:query_depth] }
                                        .uniq
                                        .max
      end

      def children(depth = -1)
        if depth == -1
          object[:children]
        elsif depth >= 0
          object[:children].select do |element|
            element[:query_depth] == depth
          end
        end
      end

      def tree
        # fonction qui itère sur chaque élément passé dans self.object, puis qui structure les éléments présents
        # dans chacun de leur champ d'agrégation children
        # retourne à la fin un mapping de l'arborescence structurée de l'arbre sous forme de liste
        #   le champ query_depth des children indique le niveau de profondeur de l'objet en rapport avec le document mongo
        #   on peut utiliser le champ parent_id et id des children pour gérer l'imbriquation des éléments
        # l'idée étanobjectt d'imbriquer de façon récursive ce qui est possible d'intégrer dans un champ children
        # exemple de retour :
        # [
        #   {id: n0, title: etp1, ..., children: [
        #     {id: n1, title: etp2, ..., children: [
        #       {id: n2, title: etp3, ..., children: [ ... ]}
        #     ]},
        #     {id: n3, title: etp4, ..., children: [
        #       {id: n4, title: etp5, ..., children: [ ... ]}
        #     ]}
        #   ]},
        #   {id: n5, title: etpobject6, ..., children: [
        #     {id: n6, title: etp7, ..., children: [
        #       {id: n7, title: etp8, ..., children: [ ... ]}
        #     ]},
        #     {id: n8, title: etp9, ..., children: [
        #       {id: n9, title: etp10, ..., children: [ ... ]}
        #     ]}
        #   ]},
        #   ...
        # ]
        children(0).map do |direct_child|
          recursive_serialize(direct_child)
        end
      end
    end
  end
end
